// Acceptor created Jul 11, 2002 8:18:34 AM
// Copyright (c) 2002, Ward Cunningham

import java.lang.reflect.Method;
import java.io.*;

public class Acceptor {
    static int right = 0;
    static int wrong = 0;
    static int ignores = 0;
    static int exceptions = 0;
    static File input;

    // main /////////////////////////////////////

    public static void main (String argv[]) throws Exception {
        if (argv.length != 2) {
            System.err.println("usage: java Acceptor input-file output-file");
            System.exit(0);
        }
        Parse tables = parse(argv[0]);
        Acceptor results = new Acceptor();
        results.doTables(tables);
        report(argv[1], tables, results);
        System.exit(results.wrong > 0 || results.exceptions >0 ? 1 : 0);
    }

    static Parse parse(String fileName) throws IOException {
        input = new File(fileName);
        char chars[] = new char[(int)(input.length())];
        FileReader in = new FileReader(fileName);
        in.read(chars);
        in.close();
        return new Parse(new String(chars));
    }

    static void report(String fileName, Parse tables, Acceptor results) throws IOException {
        PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));
        tables.print(out);
        System.err.println(
                results.right + " right, " +
                results.wrong + " wrong, " +
                results.ignores + " ignored, " +
                results.exceptions + " exceptions");
        out.close();
    }
    // table traversal //////////////////////////

    void doTables(Parse tables) {
        while (tables != null) {
            doTable(tables);
            tables = tables.more;
        }
    }
    void doTable(Parse table) {
        Parse heading = table.at(0,0,0);
        if (heading != null) {
            try {
                Acceptor head = (Acceptor)(Class.forName(heading.body).newInstance());
                head.doRows(table.parts);
            } catch (Exception e) {
                exception (heading, e);
            }
        }
    }

    void doRows(Parse rows) {
        rows = rows.more;
        while (rows != null) {
            doCells(rows.parts);
            rows = rows.more;
        }
    }

    void doCells(Parse cells) {
        for (int i=0; cells != null; i++) {
            try {
                doCell(cells, i);
            } catch (Exception e) {
                exception(cells, e);
            }
            cells=cells.more;
        }
    }

    void doCell(Parse cell, int columnNumber) {
    }

    // format converters ////////////////////////

    long parseLong (Parse cell) {
        return Long.parseLong(cell.body);
    }

    double parseDouble (Parse cell) {
        return Double.parseDouble(cell.body);
    }

    // answer comparisons ///////////////////////

    void equals (Parse cell, String value) {
        if (cell.body.equals(value)) {
			right(cell);
		} else {
            wrong(cell, value);
        }
    }

    void equals (Parse cell, long value) {
        if (parseLong(cell) == value) {
			right(cell);
		} else {
            wrong(cell, Long.toString(value));
        }
    }

    void equals (Parse cell, double value) {
        if (Double.parseDouble(cell.body) == value) {
			right(cell);
		} else {
            wrong(cell, Double.toString(value));
        }
    }

    // annotation ///////////////////////////////

    void right (Parse cell) {
		cell.addToTag(" bgcolor=\"#cfffcf\"");
		right++;
	}

    void wrong (Parse cell, String expected) {
        cell.addToBody("<hr>" + expected);
        cell.addToTag(" bgcolor=\"#ffcfcf\"");
        wrong++;
    }

    void ignore (Parse cell) {
        cell.addToTag(" bgcolor=\"#cfcfcf\"");
        ignores++;
    }

    void exception (final Parse cell, final Exception exception) {
        final StringWriter buf = new StringWriter();
        exception.printStackTrace(new PrintWriter(buf));
        cell.addToBody("<hr><font size=-2>" + br(buf.toString()) + "</font>");
        cell.addToTag(" bgcolor=\"#ffffcf\"");
        exceptions++;
    }

    String br (String text) {
        for (int i = text.indexOf('\n')+1; i>0; i=text.indexOf('\n',i)+1) {
            text = text.substring(0, i) + "<br>" + text.substring(i);
        }
        return text;
    }
}
