// Parse created Jul 10, 2002 9:44:11 PM
// Copyright (c) 2002, Ward Cunningham

import java.io.FileReader;
import java.io.PrintWriter;
import java.util.GregorianCalendar;

public class Parse {

    String leader;
    String tag;
    String body;
    String end;
    String trailer;

    Parse more;
    Parse parts;

    static String tags[] = {"table", "tr", "td"};

    Parse (String text) {
        this (text, tags, 0);
    }

    Parse (String text, String tags[], int level) {

        int startTag = text.indexOf("<"+tags[level]);
        int endTag = text.indexOf(">", startTag) + 1;
        int startEnd = text.indexOf("</"+tags[level], endTag);
        int endEnd = text.indexOf(">", startEnd) + 1;
        int startMore = text.indexOf("<"+tags[level], endEnd);

        leader = text.substring(0,startTag);
        tag = text.substring(startTag, endTag);
        body = text.substring(endTag, startEnd);
        end = text.substring(startEnd,endEnd);
        trailer = text.substring(endEnd);

        if (startMore>=0) {
            more = new Parse (trailer, tags, level);
            trailer = null;
        }

        if (level+1 < tags.length) {
            parts = new Parse (body, tags, level+1);
            body = null;
        }
    }

    public Parse at (int i) {
        Parse result = this;
        while (i-- > 0) {
            if (result != null) {
                result = result.more;
            }
        }
        return result;
    }

    public Parse at (int i, int j) {
        Parse element = this.at(i);
        if (element != null && element.parts != null) {
            return element.parts.at(j);
        } else {
            return null;
        }
    }

    public Parse at (int i, int j, int k) {
        Parse element = this.at(i, j);
        if (element != null && element.parts != null) {
            return element.parts.at(k);
        } else {
            return null;
        }
    }

    public void addToTag(String text) {
        int last = tag.length()-1;
        tag = tag.substring(0,last) + text + ">";
    }

    public void addToBody(String text) {
        body = body + text;
    }

    void print(PrintWriter out) {
        out.print(leader);
        out.print(tag);
        if (parts != null) {
            parts.print(out);
        } else {
            out.print(body);
        }
        out.print(end);
        if (more != null) {
            more.print(out);
        } else {
            out.print(trailer);
        }
    }

    public static void main (String argv[]) throws Exception {
        byte b[] = new byte[100000];
        System.in.read(b);
        Parse p = new Parse(new String(b));
        p.at(1,3,2).addToTag(" bgcolor=#cfffcf");
        p.at(1,4,2).addToTag(" bgcolor=#ffcfcf");
        p.at(1,4,2).addToBody("<hr>Null Pointer Exception");
        p.print(new PrintWriter(System.out));
    }
}
